package com.elsevier.example.music.quarkus.writer;

import com.elsevier.example.music.quarkus.ChartEntry;
import software.amazon.awssdk.services.dynamodb.DynamoDbClient;
import software.amazon.awssdk.services.dynamodb.model.*;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static java.time.format.DateTimeFormatter.ISO_LOCAL_DATE;

public class DynamoChartEntryWriter implements ChartEntryWriter {

    private final DynamoDbClient dynamoDbClient;
    private final int batchSize;
    private final String tableName;
    private List<ChartEntry> currentBatch;

    public DynamoChartEntryWriter(DynamoDbClient dynamoDbClient, String tableName, int batchSize) {
        this.dynamoDbClient = dynamoDbClient;
        this.tableName = tableName;
        this.batchSize = batchSize;
        this.currentBatch = new ArrayList<>(batchSize);
    }

    public void write(ChartEntry chartEntry) {
        currentBatch.add(chartEntry);
        if (currentBatch.size() >= batchSize) {
            writeCurrentBatch();
        }
    }

    @Override
    public void close() {
        if (currentBatch.size() > 0) {
            writeCurrentBatch();
        }
    }

    private void writeCurrentBatch() {
        BatchWriteItemRequest request = BatchWriteItemRequest.builder()
                .requestItems(Map.of(tableName, buildWriteRequestsOfCurrentBatch()))
                .build();
        dynamoDbClient.batchWriteItem(request);
        currentBatch.clear();
    }

    private Collection<WriteRequest> buildWriteRequestsOfCurrentBatch() {
        return currentBatch.stream()
                .map(DynamoChartEntryWriter::toWriteRequest)
                .collect(Collectors.toList());
    }

    private static WriteRequest toWriteRequest(ChartEntry chartEntry) {
        String artistAndSong = String.format("%s - %s", chartEntry.getArtist(), chartEntry.getSong());
        String dateString = ISO_LOCAL_DATE.format(chartEntry.getDate());
        String positionString = String.valueOf(chartEntry.getPosition());
        Map<String, AttributeValue> attributeMap = Map.of(
                "ArtistSong", AttributeValue.builder().s(artistAndSong).build(),
                "Date", AttributeValue.builder().s(dateString).build(),
                "Position", AttributeValue.builder().n(positionString).build()
        );

        return WriteRequest.builder()
                .putRequest(PutRequest.builder()
                        .item(attributeMap)
                        .build())
                .build();
    }
}
